/***************************************************************************
 *   Copyright (C) 2004 by Stefano                                         *
 *   stefano@xiaprojects.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "xeconfiguration.h"
#include "xeobject.h"
#include <kstandarddirs.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <qdir.h>
#include <qlibrary.h>

#include <stdlib.h>

#define ENABLE_FINAL


XEConfiguration::XEConfiguration(QObject *parent, const char *name)
 : QObject(parent, name)
{
	XEObject::xSetEngineConfiguration(this);
	
	xReset();
}


XEConfiguration::~XEConfiguration()
{
}


#include "xeconfiguration.moc"


/*!
    \fn XEConfiguration::xReset()
 */
void XEConfiguration::xReset()
{

	ActiveConfiguration.AutoSave=1;
	ActiveConfiguration.Version=KXDOCKERVERSION;
	ActiveConfiguration.Icons.Size=32;
	ActiveConfiguration.Icons.SizeBig=128;
	ActiveConfiguration.Icons.Separation=5;
	ActiveConfiguration.Icons.Raise=-50;
	ActiveConfiguration.Icons.Horiz=32;
	ActiveConfiguration.Icons.EnableThumbnail=0;
	
	ActiveConfiguration.Window.FastHide=0;
	ActiveConfiguration.Window.borderLeft=200;
	ActiveConfiguration.Window.borderTop=200;
	ActiveConfiguration.Window.Width=1024;
	ActiveConfiguration.Window.Height=300;
	ActiveConfiguration.Window.Top=100;
	ActiveConfiguration.Window.Left=0;
	ActiveConfiguration.Window.HeightDesktop=0;
	ActiveConfiguration.Window.LeftForce=0;
	ActiveConfiguration.Window.TopForce=0;
	
	ActiveConfiguration.Window.HideTimeout=0;
	ActiveConfiguration.Window.HideMouseEdge=1;
	
	
	
	ActiveConfiguration.Window.SendToForgroundTimeout=500;
	ActiveConfiguration.Window.RaiseOnEvents=0;
	
	// 0.27
	ActiveConfiguration.Matrix.XMatrix=NULL;
	ActiveConfiguration.Matrix.YMatrix=NULL;
	ActiveConfiguration.Matrix.ZMatrix=NULL;
	ActiveConfiguration.Matrix.KMatrix=NULL;
	ActiveConfiguration.Matrix.DMatrix=NULL;
	
	ActiveConfiguration.Matrix.DesignDMoltiplier=1;
	ActiveConfiguration.Matrix.DesignYMoltiplier=1;
	ActiveConfiguration.Matrix.DesignXMoltiplier=1;
	ActiveConfiguration.Matrix.Design="panther";

	
	
	ActiveConfiguration.Engine.SmoothTimeout=200;
	//strcpy(ActiveConfiguration.Engine.XMLConf,"kxdocker_conf.xml");
	KConfig *cfg=KGlobal::config();
	//cfg->setGroup("");
	ActiveConfiguration.Engine.XMLConf=cfg->readEntry("KXDockerXMLConf","kxdocker/kxdocker_conf.xml");
	if(ActiveConfiguration.Engine.XMLConf=="" 
		||ActiveConfiguration.Engine.XMLConf==NULL
		||ActiveConfiguration.Engine.XMLConf==QString::null)
	{
		ActiveConfiguration.Engine.XMLConf="kxdocker/kxdocker_conf.xml";
	}
	ActiveConfiguration.Engine.XMLConf=::locate("data",ActiveConfiguration.Engine.XMLConf);
#ifndef ENABLE_FINAL
	qWarning(ActiveConfiguration.Engine.XMLConf);
#endif
	QFile a(ActiveConfiguration.Engine.XMLConf);
	if(!a.exists() || ActiveConfiguration.Engine.XMLConf==QString::null)
	{
		QString cfgPath="kxdocker/kxdocker_conf.xml";
		ActiveConfiguration.Engine.XMLConf=::locate("data",cfgPath);
		if(ActiveConfiguration.Engine.XMLConf==NULL)
		{
			KFileDialog fd(QString::null,QString::null,0,"openXMLConf",true);
			cfgPath=fd.getOpenFileName(QString::null,QString::null,0,"Open KXDocker XML Configuration");
			ActiveConfiguration.Engine.XMLConf=::locate("data",cfgPath);
			if(ActiveConfiguration.Engine.XMLConf==NULL)
			{
				qWarning("cannot find kxdocker_conf.xml");
				exit(1);
			}
		}
		cfg->writeEntry("KXDockerXMLConf",ActiveConfiguration.Engine.XMLConf);
	}
	
	// checkout if resources are correctly installed
	QString ResourcesPath=QString("kxdocker/kxdocker_resources-%1").arg(KXDOCKERNEED);
	ResourcesPath=::locate("data",ResourcesPath);
	if(!a.exists() || ResourcesPath==QString::null)
	{
		qWarning("You may need to update or reinstall KXDocker resources, checkout http://www.xiaprojects.com/www/prodotti/kxdocker/main.php?action=download#resources");
	}
	
	ActiveConfiguration.Engine.SleepFPS=100;
	ActiveConfiguration.Engine.SleepThreads=1000;
	ActiveConfiguration.Engine.SleepAnimations=1000;
	ActiveConfiguration.Engine.MaxIconsShowed=MAX_ICONS;
	ActiveConfiguration.Engine.AndZoomCache=0;
	
	ActiveConfiguration.Window.Align="bottom";

	QString backgroundPath="kxdocker/themes/bar/default/";
	ActiveConfiguration.Theme.ArrowsPath="kxdocker/themes/arrows/";
	
	ActiveConfiguration.Theme.BackgroundPath=::locate("data",backgroundPath);
	if(ActiveConfiguration.Theme.BackgroundPath==NULL)
	{
#ifndef ENABLE_FINAL
		qWarning("Cannot find background themes!");
#endif
		ActiveConfiguration.Theme.BackgroundPath="kxdocker/themes/bar/default";
	}
	ActiveConfiguration.Theme.Background.Tiled=1;
	ActiveConfiguration.Theme.Background.Desaturate=0;
	ActiveConfiguration.Theme.Background.imgBackgroundCenter="background-center.png";
	ActiveConfiguration.Theme.Background.imgBackgroundLeft="background-left.png";
	ActiveConfiguration.Theme.Background.imgBackgroundRight="background-right.png";
	ActiveConfiguration.Theme.Background.imgBackgroundOver="background-over.png";
	
	ActiveConfiguration.Plugins.PluginsPaths.clear();
	
#ifdef ENABLE_SPECIALENGINE2
	ActiveConfiguration.Engine.InterpolationValue=2;
#endif
	ActiveConfiguration.Window.SendToBackground=1;
	
	ActiveConfiguration.ConfigurationsPaths.clear();
	ActiveConfiguration.ConfigurationsPaths.append("kxdocker/xml/");
}


/*!
    \fn XEConfiguration::xGetConfiguration()
 */
XSConfigurations * XEConfiguration::xGetConfiguration()
{
	return &ActiveConfiguration;
}


/*!
    \fn XEConfiguration::xCfgDefault()
 */
void XEConfiguration::xCfgDefault()
{
    /// @todo implement me
}


/*!
    \fn XEConfiguration::xSetup()
 */
void XEConfiguration::xSetup()
{
    /// @todo implement me
}


/*!
    \fn XEConfiguration::iconRemove(int)
 */
void XEConfiguration::iconRemove(int icon)
{
	// removing all dead info
	ActiveConfiguration.ObjectsIcons.remove(icon);
	return;
}


/*!
    \fn XEConfiguration::xLoadModules();
 */
void XEConfiguration::xLoadModules()
{
	for(unsigned int i=0;i<ActiveConfiguration.ObjectsIcons.count();i++)
	{
		if(ActiveConfiguration.ObjectsIcons.at(i)->Info.className!="GIcon")
		{
			QObject *GClass
				=
				XEObject::xFindObject(
					ActiveConfiguration.ObjectsIcons.at(i)->Info.className
				);
			if(GClass==NULL)
			{
#ifndef ENABLE_FINAL
				qWarning("xeconfiguration.cpp:194 Module not found, try to load it...");
#endif
				xLoadModule(ActiveConfiguration.ObjectsIcons.at(i)->Info.className);
			}
			else
			{
#ifndef ENABLE_FINAL
				qWarning("xeconfiguration.cpp:199 Module FOUND!");
#endif
			}
		}
	}
}

typedef QObject *(*pPluginGenerator)(QObject *parent);

/*!
    \fn XEConfiguration::xLoadModule(const QString moduleName)
 */
void XEConfiguration::xLoadModule(const QString moduleName)
{
#ifndef ENABLE_FINAL
	qWarning("xeconfiguration.cpp:212 try to loading a new module");
	qWarning(moduleName);
#endif
	for(unsigned int i=0; i<ActiveConfiguration.Plugins.PluginsPaths.count();i++)
	{
		pPluginGenerator PluginGenerator;
		
		if (ActiveConfiguration.Plugins.PluginsPaths[i]
			[ActiveConfiguration.Plugins.PluginsPaths[i].length()-1]!='/')
				ActiveConfiguration.Plugins.PluginsPaths[i]=
				ActiveConfiguration.Plugins.PluginsPaths[i]+"/";

		QString SOName=::locate("data",ActiveConfiguration.Plugins.PluginsPaths[i]);
		SOName=SOName+moduleName+
		//ActiveConfiguration.Plugins.SOName;
		".so";
#ifndef ENABLE_FINAL
		qWarning(SOName);
#endif
PluginGenerator=(pPluginGenerator)QLibrary::resolve(SOName,"xeplugin_register");
		if(PluginGenerator)
		{
#ifndef ENABLE_FINAL
			qWarning("found the right so, now running it");
#endif
			//QObject *LoadedModule=PluginGenerator(parent());
			//QObject *LoadedModule=PluginGenerator(XEObject::xGetDocker());
			QObject *LoadedModule=PluginGenerator(XEObject::xGetRoot());
			if(LoadedModule)
			{
#ifndef ENABLE_FINAL
qWarning("yea i've loaded it!!");
qWarning(LoadedModule->name());
#endif
			XEObject::xPluginAdd(LoadedModule);
			}
			return;
		}
		else
		{
#ifndef ENABLE_FINAL
			qWarning("NOT found the right so");
#endif
		}
	}
}


/*!
    \fn XEConfiguration::pluginRemove(const QString)
 */
void XEConfiguration::pluginRemove(const QString removeName)
{
	for(uint k=0;k<ActiveConfiguration.Plugins.Plugin.count();k++)
	{
		if(ActiveConfiguration.Plugins.Plugin.at(k)->Info.Name=removeName)
		{
			ActiveConfiguration.Plugins.Plugin.remove(k);
			return;
		}
	}
}


/*!
    \fn XEConfiguration::pluginAdd(const QString)
 */
void XEConfiguration::pluginAdd(const QString newName)
{
	XSGObjectPlugin	*newPlugin=new XSGObjectPlugin();
	newPlugin->Info.Name=newName;

			// Creating fake document
			QDomDocument 	doc( "KXDocker_Conf" );
			// the fake root
			QDomElement fakeRoot=doc.createElement("FakeRoot");
			// appending childs
			doc.appendChild(fakeRoot);
			// now creating the configuration
			QDomElement CreatingCfg=doc.createElement("pluginconf");
			
			// Now appending to the configuration
			fakeRoot.appendChild(CreatingCfg);
			newPlugin->xmlConfiguration=fakeRoot.childNodes();//fa	
	

	ActiveConfiguration.Plugins.Plugin.append(newPlugin);
}


/*!
    \fn XEConfiguration::xGetConfiguration(XSConfiguration *&)
 */
void XEConfiguration::xGetConfiguration(void *&a,const QString Version, int SizeOf)
{
a=NULL;
if(Version==KXDOCKERHEADERVERSION && SizeOf==sizeof(ActiveConfiguration))a=&ActiveConfiguration;
}


/*!
    \fn XEConfiguration::xGetConfigurationVersion(int &, QString)
 */
void XEConfiguration::xGetConfigurationVersion(int &i, QString &s)
{

s=KXDOCKERHEADERVERSION;
i=sizeof(ActiveConfiguration);

}


/*!
    \fn XEConfiguration::xGetConfigurationValue(const QString, QStringList &)
 */
void XEConfiguration::xGetConfigurationValue(const QString, QStringList &)
{
    /// @todo implement me
}


/*!
    \fn XEConfiguration::xGetConfigurationValue(const QString, int &)
 */
void XEConfiguration::xGetConfigurationValue(const QString vpath, int &vvalue)
{	
	int *pV=(int *)xGetConfigurationPointer(vpath);
	if(pV!=NULL)vvalue=*pV;
	else vvalue=0;
}


/*!
    \fn XEConfiguration::xGetConfigurationValue(const QString, QString &)
 */
void XEConfiguration::xGetConfigurationValue(const QString vpath, QString &vvalue)
{
	
	QString *pV=(QString *)xGetConfigurationPointer(vpath);
	if(pV!=NULL)vvalue=*pV;
	else vvalue="";
}


/*!
    \fn XEConfiguration::xGetConfigurationPointer(const QString)
 */
void * XEConfiguration::xGetConfigurationPointer(const QString vpath)
{
	QStringList lpath=QStringList::split ("/",vpath,FALSE);
	if(lpath[0]=="icons")
	{
		if(lpath[1]=="size")return &ActiveConfiguration.Icons.Size;
		if(lpath[1]=="sizebig")return &ActiveConfiguration.Icons.SizeBig;
		if(lpath[1]=="sizeprefer")return &ActiveConfiguration.Icons.SizePrefer;
	}
	return NULL;
}


/*!
    \fn XEConfiguration::iconAdd()
 */
XSGObjectIcon * XEConfiguration::iconAdd()
{
			// 0.35
	XSGObjectIcon *addedCfg=new XSGObjectIcon;

	//addedCfg->Actions.ActionList.clear();
	addedCfg->Actions.onClickExec="";
	addedCfg->Actions.onDropExec="";
	addedCfg->Images.imgFileArrow="arrow.png";
	addedCfg->Images.imgFileDrop="drop.png";
	addedCfg->Images.imgFileName="default";
	addedCfg->Info.Group=addedCfg->Info.Name=QString("Added%1").arg((long)addedCfg);
	addedCfg->Info.className="GIcon";
	addedCfg->Info.fileName="";
	addedCfg->OverText="";
	//addedCfg->PluginConfiguration
	addedCfg->PluginPopupMenu=NULL;
	addedCfg->PoofName="poof.png";
	addedCfg->Tasks.ClassName.clear();
	addedCfg->Tasks.PopupMenu=NULL;
	addedCfg->Tasks.Proces.clear();
	addedCfg->Tasks.ShowThumbnail=0;
	addedCfg->Tasks.TaskInfo=NULL;
	addedCfg->Tasks.TaskName.clear();
	addedCfg->Tasks.Thumbnail.reset();
	addedCfg->Tasks.WindowTitle.clear();
	addedCfg->Tasks.dcopClose="";
	addedCfg->Tasks.dcopName.clear();
	addedCfg->Tasks.dcopNewWindow="";
	addedCfg->Tasks.dcopReady=0;
	addedCfg->Tasks.dcopRestore="";
	addedCfg->Tasks.forceRunning=0;

	return addedCfg;

}
